/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*******************************************************************************
* MPIA, NEVEC - MIDI project
*
* "@(#) $Id: oi_target.c,v 1.3 2010/01/26 10:01:42 jaffe Exp $"
*
* who        when      what
* ---------  --------  ----------------------------------------------
* jaffe      2010-01-22 created
*
*/

#define _POSIX_SOURCE_1
#include "vltPort.h"

/* $Id: oi_target.c,v 1.3 2010/01/26 10:01:42 jaffe Exp $ */

/*----------------------------------------------------------------------- */
/* TARGET data FITS binary tables routines                            */
/* target definition data for OI_FITS environment                     */
/*----------------------------------------------------------------------- */
#include "oi_target.h"

/* internal prototypes */
/* get data from table_def specific to target table */
void get_target (target_table_def *table);
/* file global variables for table type */
static char *tabletype = "OI_TARGET";


/* target_row functions */
target_table_row* make_target_table_row(target_table_def* table)
/*----------------------------------------------------------------------- */
/*   Make an target_table_row structure                                   */
/*   Inputs:                                                              */
/*      table     pointer to target table structure                 */
/*   Returns:                                                             */
/*      An target_table_row structure pointer ; NULL indicates failure    */
/*      Data pointers set to generic structure values                     */
/*----------------------------------------------------------------------- */
{
  target_table_row *out;
  int i, icol, nregion;

  /* create basic structure */
  out = (target_table_row*)malloc(sizeof(target_table_row));
  if (out==NULL) return out; /* allocation failure? */
  
  /* zero memory */
  memset (out, 0, sizeof(target_table_row));
 

  /* Make generic structure */
  out->raw = make_table_row_from_def(table->def);

  return out; /* done */
} /* end  make_target_table_row */

void kill_target_table_row (target_table_row *row)
/*----------------------------------------------------------------------- */
/*   Delete an target_table_row structure                                   */
/*   Inputs:                                                              */
/*      row    pointer to the target_table_row structure.                   */
/*----------------------------------------------------------------------- */
{
  int i, nregion;
  if (row==NULL) return; /* sanity check */

  free (row->raw); /* generic structure */
  free (row);      /* basic structure */
} /* end  kill_target_table_row */


/* target_table functions */
target_table_def*
make_target_table_def(fitsfile *fptr, int *ierr)
/*----------------------------------------------------------------------- */
/*   Make a new target_table in an open FITS file                            */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      An target_table_def pointer; NULL indicates failure                  */
/*----------------------------------------------------------------------- */
{
  target_table_def *out = NULL;
  int    version = 1; /* table version number */
  int    ikey, icol, i;

  if (*ierr>0) return NULL;    /* existing error condition? */

  if (fptr==NULL) return NULL; /* sanity check */

  /* create output object */
  out = (target_table_def*)malloc(sizeof(target_table_def));
  if (out==NULL)     {  /* check for allocation failure */
    *ierr = 10000;
    fprintf(stderr,"ERROR creating %s table_def \n", tabletype);
    return out;
  }
  memset (out, 0, sizeof(target_table_def)); /* zero fill */

  /* create table_def */
  out->def = make_table_def(OI_TARGET_TABLE, 0, 0);
  if (out->def==NULL) { /* check for failure */
    kill_target_table_def(out); /* clean up */
    *ierr = 10000;
    fprintf(stderr,"ERROR creating OI_TARGET_TABLE for %s table_def \n", 
	    tabletype);
    return NULL;
  }

  /* add keywords */
  ikey = table_find_key(out->def, "OI_REVN");
  if ((ikey>0) && (*ierr==0)) table_put_keyword (out->def, ikey, 
						(void*)&version, ierr);

  /* get info from table descriptor */
  get_target_info (out);

  /* open imaging data table */
  make_table_from_def (fptr, out->def, ierr);

  return out; /* done */
} /* end make_target_table */

target_table_def* open_target_table(fitsfile *fptr, int *ierr)
/*----------------------------------------------------------------------- */
/*   Open an existing target_table in the active FITS file            */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      An target_table_def pointer; NULL indicates failure           */
/*----------------------------------------------------------------------- */
{
  target_table_def *out = NULL;
  int i,iKey;
  char kString[20], typeString[20];
  char *tForm = "TFORM";

  if (*ierr>0) return NULL;    /* existing error condition? */
  if (fptr==NULL) return NULL; /* sanity check */

  /* create output object */
  out = (target_table_def*)malloc(sizeof(target_table_def));
  if (out==NULL)     {  /* check for allocation failure */
    *ierr = 10000;
    fprintf(stderr,"ERROR creating %s table_def \n", tabletype);
    return out;
  }
  memset (out, 0, sizeof(target_table_def)); /* zero fill */

  /* position FITS file at  table */
  if (fits_movnam_hdu(fptr, BINARY_TBL, tabletype, 0, ierr)) {
    fprintf(stderr,"ERROR %d finding %s table \n", *ierr, tabletype);
    kill_target_table_def(out); /* clean up */
    *ierr = 10000;
    return NULL;
  }

  /* get table descriptor */
  out->def = make_table_def_from_table (fptr);
  if (out->def==NULL) { /* error check */
    fprintf(stderr,"ERROR making table def for %s \n", tabletype);
    kill_target_table_def(out); /* clean up */
    *ierr = 10000;
    return NULL;
  }

  /* copy info from table descriptor */
  get_target_info (out);

  return out; /* done */
} /* end open_target_table */

int read_target_table(fitsfile *fptr, target_table_def* table, 
			    target_table_row *row, long row_number,
			    int *ierr)
/*----------------------------------------------------------------------- */
/*   Read the next row from an target_table                           */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      table     table definitions structure from open or create         */
/*      row_number which row (1-rel) in the table                         */   
/*                 -1 => next row                                         */  
/*   Input/output:                                                        */
/*      row       pointer to an extant row structure                      */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */
{
  table_row *trow;
  int        i;

  if (*ierr>0) return 1;     /* existing error condition? */
  if (fptr==NULL) return 1;  /* sanity check */
  if (table==NULL) return 1; /* sanity check */
  if (row==NULL) return 1;   /* sanity check */

  /* set row number */
  if (row_number>0) { /* explicit row number */
    row->raw->trow = row_number;
    table->row = row_number;
  } else { /* next row */
    row->raw->trow = ++table->row;
  }

  /* read row */
  table_read_row(fptr, table->def, row->raw, ierr);
  if (*ierr>0) { /* error check */
    fprintf(stderr,"ERROR  read_target_table: reading table %s \n", 
	    tabletype);
    return 1;
  }

  /* copy relevant data from generic table_row */
  trow = row->raw;
  row->target_id   = trow->data[table->target_id_col]->tdata.tshort[0];
  strcpy(row->target, trow->data[table->target_col]->tdata.tchar);
  row->raep0       = trow->data[table->raep0_col]->tdata.tdouble[0];
  row->decep0      = trow->data[table->decep0_col]->tdata.tdouble[0];
  row->equinox     = trow->data[table->equinox_col]->tdata.tfloat[0];
  row->ra_err      = trow->data[table->ra_err_col]->tdata.tdouble[0];
  row->dec_err     = trow->data[table->dec_err_col]->tdata.tdouble[0];
  row->sysvel      = trow->data[table->sysvel_col]->tdata.tdouble[0];
  strcpy(row->veltyp, trow->data[table->veltyp_col]->tdata.tchar);
  strcpy(row->veldef, trow->data[table->veldef_col]->tdata.tchar);
  row->pmra        = trow->data[table->pmra_col]->tdata.tdouble[0];
  row->pmdec       = trow->data[table->pmdec_col]->tdata.tdouble[0];
  row->pmra_err    = trow->data[table->pmra_err_col]->tdata.tdouble[0];
  row->pmdec_err   = trow->data[table->pmdec_err_col]->tdata.tdouble[0];
  row->parallax    = trow->data[table->parallax_col]->tdata.tfloat[0];
  row->para_err    = trow->data[table->para_err_col]->tdata.tfloat[0];
  strcpy(row->spectyp, trow->data[table->spectyp_col]->tdata.tchar);
  return 0; /* OK - done */
} /* end read_target_table */

int write_target_table(fitsfile *fptr, target_table_def* table, 
			     target_table_row *row, long row_number, 
			     int *ierr)
/*----------------------------------------------------------------------- */
/*   Write  the next row to an target_table                         */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      table     table definitions structure from open or create         */
/*      row       pointer to an extant row structure                      */
/*      row_number which row (1-rel) in the table                         */     
/*                 -1 => next row                                         */  
/*   Input/output:                                                        */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */
{
  table_row *trow;
  int        i;

  if (*ierr>0) return 1;     /* existing error condition? */
  if (fptr==NULL) return 1;  /* sanity check */
  if (table==NULL) return 1; /* sanity check */
  if (row==NULL) return 1;   /* sanity check */

  /* set row number */
  if (row_number>0) { /* explicit row number */
    row->raw->trow = row_number;
    table->row = row_number;
  } else { /* next row */
    row->raw->trow = ++table->row;
  }

  /* copy relevant data to generic table_row */
  trow = row->raw;
  trow->data[table->target_id_col]->tdata.tshort[0] = row->target_id;
  strcpy(trow->data[table->target_col]->tdata.tchar, row->target);
  trow->data[table->raep0_col]->tdata.tdouble[0]    = row->raep0;
  trow->data[table->decep0_col]->tdata.tdouble[0]   = row->decep0;
  trow->data[table->equinox_col]->tdata.tfloat[0]   = row->equinox;
  trow->data[table->ra_err_col]->tdata.tdouble[0]   = row->ra_err;
  trow->data[table->dec_err_col]->tdata.tdouble[0]  = row->dec_err;
  trow->data[table->sysvel_col]->tdata.tdouble[0]   = row->sysvel;
  strcpy(trow->data[table->veltyp_col]->tdata.tchar, row->veltyp);
  strcpy(trow->data[table->veldef_col]->tdata.tchar, row->veldef);
  trow->data[table->pmra_col]->tdata.tdouble[0]     = row->pmra;
  trow->data[table->pmdec_col]->tdata.tdouble[0]    = row->pmdec;
  trow->data[table->pmra_err_col]->tdata.tdouble[0] = row->pmra_err;
  trow->data[table->pmdec_err_col]->tdata.tdouble[0]= row->pmdec_err;
  trow->data[table->parallax_col]->tdata.tfloat[0]  = row->parallax;
  trow->data[table->para_err_col]->tdata.tfloat[0]  = row->para_err;
  /* write row */
  table_write_row(fptr, table->def, row->raw, ierr);
  if (*ierr>0) { /* error check */
    fprintf(stderr,"ERROR  write_target_table: writing table %s \n", 
	    tabletype);
    return 1;
  }

  return 0; /* OK - done */
} /* end write_target_table */

int close_target_table(fitsfile *fptr, target_table_def* table, 
			     int *ierr)
/*----------------------------------------------------------------------- */
/*   Close an open target_table, associated structures are destroyed  */
/*   Note: this closes the full FITS file                                 */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */
{
  if (*ierr>0) return 1;    /* existing error condition? */
  if (fptr==NULL) return 1; /* sanity check */

  /* do actual file close */
  fits_close_file (fptr, ierr);

  /* delete structures */
  kill_target_table_def(table);

  return 0; /* OK - done */
} /* end close_target_table */

void kill_target_table_def (target_table_def* table)
/*----------------------------------------------------------------------- */
/*   Delete an target_table structure                                 */
/*   Inputs:                                                              */
/*      table     table definitions structure from open or create         */
/*----------------------------------------------------------------------- */
{
  if (table==NULL) return; /* sanity check */

  /* table definition structure */
  kill_table_def(table->def);

  /* free basic structure */
  free (table);
} /* end kill_target_table_def */

/*************************private functions ******************************/

void get_target_info (target_table_def *table)
/*----------------------------------------------------------------------- */
/*   Get data from table_def specific to target table                 */
/*   Inputs:                                                              */
/*      table     table definitions structure                             */
/*----------------------------------------------------------------------- */
{
  if (table==NULL) return;         /* sanity check */

  /* lookup column numbers (0-rel) */
  table->target_id_col  = table_find_col (table->def, "TARGET_ID")-1;
  table->target_col     = table_find_col (table->def, "TARGET")-1;
  table->raep0_col      = table_find_col (table->def, "RAEP0")-1;
  table->decep0_col     = table_find_col (table->def, "DECEP0")-1;
  table->equinox_col    = table_find_col (table->def, "EQUINOX")-1;
  table->ra_err_col     = table_find_col (table->def, "RA_ERR")-1;
  table->dec_err_col    = table_find_col (table->def, "DEC_ERR")-1;
  table->sysvel_col     = table_find_col (table->def, "SYSVEL")-1;
  table->veltyp_col     = table_find_col (table->def, "VELTYP")-1;
  table->veldef_col     = table_find_col (table->def, "VELDEF")-1;
  table->pmra_col       = table_find_col (table->def, "PMRA")-1;
  table->pmdec_col      = table_find_col (table->def, "PMDEC")-1;
  table->pmra_err_col   = table_find_col (table->def, "PMRA_ERR")-1;
  table->pmdec_err_col  = table_find_col (table->def, "PMDEC_ERR")-1;
  table->parallax_col   = table_find_col (table->def, "PARALLAX")-1;
  table->para_err_col   = table_find_col (table->def, "PARA_ERR")-1;
  table->spectyp_col    = table_find_col (table->def, "SPECTYP")-1;
} /* end  get_target_info */

/* */
